searchNodes=[{"doc":"A minimal filesystem-based publishing engine that supports custom adapters. It aims to provide a more flexible mechanism for file parsing and transformation. This is a fork of NimblePublisher . Overview Suppose we have a batch of article files in following format: %{ title : &quot;Hello world&quot; } -- - Body of the &quot;Hello world&quot; article . This is a * markdown * document with support for code highlighters : ` ` ` elixir IO . puts &quot;hello world&quot; ` ` ` And, we try to use CozyPublisher to publish them: use CozyPublisher , build : Article , from : Application . app_dir ( :app_name , &quot;priv/articles/**/*.md&quot; ) , as : :articles The example above will get all matched files in the given directory, call Article.build/3 for each file, passing the file path, the metadata, and the article body, and define a module attribute named @articles with all built articles returned by the Article.build/3 function. Options :build - the name of the module that will build each entry. :from - a wildcard pattern where to find all entries. :as - the name of the module attribute to store all built entries. :adapter - the adapter and its options. It allows following formats: nil - (default) equals to {CozyPublisher.Adapters.Default, []} module() - equals to {module(), keyword()} {module(), keyword()} An example Let's see a complete example using the default adapter, which has Markdown and code highlighting support. First add cozy_publisher and other required packages as dependencies: def deps do [ { :cozy_publisher , &quot;~&gt; 1.0&quot; } , { :makeup , &quot;~&gt; 1.0&quot; } , { :makeup_elixir , &quot;&gt;= 0.0.0&quot; } , { :makeup_erlang , &quot;&gt;= 0.0.0&quot; } ] end In this example, we are building a blog. Each post stays in the &quot;posts&quot; directory with the format: / posts / YEAR / MONTH - DAY - ID . md A typical blog post will look like this: # /posts/2020/04-17-hello-world.md %{ title : &quot;Hello world!&quot; , author : &quot;José Valim&quot; , tags : ~w(hello) , description : &quot;Let&#39;s learn how to say hello world&quot; } -- - This is the post . Therefore, we will define a Post struct that expects all of the fields above. We will also have a :date field that we will build from the filename. Overall, it will look like this: defmodule MyApp.Blog.Post do @enforce_keys [ :id , :author , :title , :body , :description , :tags , :date ] defstruct [ :id , :author , :title , :body , :description , :tags , :date ] def build ( path , attrs , body ) do [ year , month_day_id ] = path |&gt; Path . rootname ( ) |&gt; Path . split ( ) |&gt; Enum . take ( - 2 ) [ month , day , id ] = String . split ( month_day_id , &quot;-&quot; , parts : 3 ) date = Date . from_iso8601! ( &quot; \#{ year } - \#{ month } - \#{ day } &quot; ) struct! ( __MODULE__ , [ id : id , date : date , body : body ] ++ Map . to_list ( attrs ) ) end end Now, we are ready to define our MyApp.Blog with CozyPublisher : defmodule MyApp.Blog do alias CozyPublisher.Adapters.Default , as : DefaultAdapter alias MyApp.Blog.Post use CozyPublisher , build : Post , from : Application . app_dir ( :my_app , &quot;priv/posts/**/*.md&quot; ) , as : :posts , adapter : { DefaultAdapter , highlighters : [ :makeup_elixir , :makeup_erlang ] } # The @posts variable is first defined by CozyPublisher. # Let&#39;s further modify it by sorting all posts by descending date. @posts Enum . sort_by ( @posts , &amp; &amp;1 . date , { :desc , Date } ) # Let&#39;s also get all tags @tags @posts |&gt; Enum . flat_map ( &amp; &amp;1 . tags ) |&gt; Enum . uniq ( ) |&gt; Enum . sort ( ) # And finally export them def all_posts , do : @posts def all_tags , do : @tags end Important : Avoid injecting the @posts attribute into multiple functions, as each call will make a complete copy of all posts. For example, if you want to show define recent_posts() as well as all_posts() , DO NOT do this: def all_posts , do : @posts def recent_posts , do : Enum . take ( @posts , 3 ) Instead do this: def all_posts , do : @posts def recent_posts , do : Enum . take ( all_posts ( ) , 3 ) Other helpers You may want to define other helpers to traverse your published resources. For example, if you want to get posts by ID or with a given tag, you can define additional functions as shown below: defmodule NotFoundError do defexception [ :message , plug_status : 404 ] end def get_post_by_id! ( id ) do Enum . find ( all_posts ( ) , &amp; ( &amp;1 . id == id ) ) || raise NotFoundError , &quot;post with id= \#{ id } not found&quot; end def get_posts_by_tag! ( tag ) do case Enum . filter ( all_posts ( ) , &amp; ( tag in &amp;1 . tags ) ) do [ ] -&gt; raise NotFoundError , &quot;posts with tag= \#{ tag } not found&quot; posts -&gt; posts end end Live reloading If you are using Phoenix, you can enable live reloading by simply telling Phoenix to watch the &quot;posts&quot; directory. Open up &quot;config/dev.exs&quot;, search for live_reload: and add this to the list of patterns: live_reload : [ patterns : [ ... , ~r&quot;posts/*/.*(md)$&quot; ] ] Custom adapters By using custom adapters, you can publish any kind of files, not limited to format required by the default adapter. You may want to define a custom adapter using JSON metadata header: defmodule MyApp.Blog do use CozyPublisher , # ... adapter : { CustomAdapter , [ ] } end defmodule CustomAdapter do use CozyPublisher.Adapter @impl true def parse ( path , content , _opts ) do [ attrs , body ] = :binary . split ( content , [ &quot; \\n --- \\n &quot; ] ) { Jason . decode! ( attrs ) , body } end end You can also create adapters supporting org files, etc. Checkout CozyPublisher.Adapter for more details.","ref":"CozyPublisher.html","title":"CozyPublisher","type":"module"},{"doc":"The specification of adapters.","ref":"CozyPublisher.Adapter.html","title":"CozyPublisher.Adapter","type":"behaviour"},{"doc":"Prepares for future works, such as: checking required dependencies starting necessary applications","ref":"CozyPublisher.Adapter.html#c:init/1","title":"CozyPublisher.Adapter.init/1","type":"callback"},{"doc":"Parses the content of files. It must return: a 2 element tuple with attributes and body - {attrs, body} a list of 2 element tuple with attributes and body - [{attrs, body} | _]","ref":"CozyPublisher.Adapter.html#c:parse/3","title":"CozyPublisher.Adapter.parse/3","type":"callback"},{"doc":"Transforms the parsed body.","ref":"CozyPublisher.Adapter.html#c:transform/3","title":"CozyPublisher.Adapter.transform/3","type":"callback"},{"doc":"The attributes parsed from file.","ref":"CozyPublisher.Adapter.html#t:attrs/0","title":"CozyPublisher.Adapter.attrs/0","type":"type"},{"doc":"The body parsed from file.","ref":"CozyPublisher.Adapter.html#t:body/0","title":"CozyPublisher.Adapter.body/0","type":"type"},{"doc":"The content of file.","ref":"CozyPublisher.Adapter.html#t:content/0","title":"CozyPublisher.Adapter.content/0","type":"type"},{"doc":"The option for current adapter.","ref":"CozyPublisher.Adapter.html#t:opts/0","title":"CozyPublisher.Adapter.opts/0","type":"type"},{"doc":"The path of file.","ref":"CozyPublisher.Adapter.html#t:path/0","title":"CozyPublisher.Adapter.path/0","type":"type"},{"doc":"The default adapter used by CozyPublisher . Files with the .md or .markdown extension will be converted to HTML with Earmark . Other files will be kept as is. Required dependencies :earmark :makeup Options :earmark_options - an %Earmark.Options{} struct :highlighters - which code highlighters to use. CozyPublisher uses Makeup for syntax highlighting and you will need to add its .css classes. You can generate the CSS classes by calling Makeup.stylesheet(:vim_style, &quot;makeup&quot;) inside iex -S mix. You can replace :vim_style by any style of your choice defined here .","ref":"CozyPublisher.Adapters.Default.html","title":"CozyPublisher.Adapters.Default","type":"module"}]